@@ -20,6 +20,7 @@ from urllib.request import urlopen
 import errno
 import logging
 import multiprocessing
+import operator
 import os
 import pkg_resources
 import random
@@ -66,10 +67,14 @@ def clear_here():
         else:
             shutil.rmtree(name)
 
-def ls(dir, *subs):
+def ls(dir, *subs, lowercase_and_sort_output=False):
     if subs:
         dir = os.path.join(dir, *subs)
-    names = sorted(os.listdir(dir))
+    if lowercase_and_sort_output:
+        # Get the original names, but sorted lowercase.
+        names = sorted(os.listdir(dir), key=operator.methodcaller("lower"))
+    else:
+        names = sorted(os.listdir(dir))
     for name in names:
         # If we're running under coverage, elide coverage files
         if os.getenv("COVERAGE_PROCESS_START") and name.startswith('.coverage.'):
@@ -80,6 +85,8 @@ def ls(dir, *subs):
             print_('l ', end=' ')
         else:
             print_('- ', end=' ')
+        if lowercase_and_sort_output:
+            name = name.lower()
         print_(name)
 
 def mkdir(*path):
@@ -179,6 +186,10 @@ def _runsetup(setup, *args):
             env=dict(os.environ,
                      PYTHONPATH=zc.buildout.easy_install.pip_pythonpath,
                      ),
+            # Prevent showing several lines of output per created distribution,
+            # especially with older setuptools versions.
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.DEVNULL,
             )
         if os.path.exists('build'):
             rmtree('build')
@@ -186,7 +197,7 @@ def _runsetup(setup, *args):
         os.chdir(here)
 
 def sdist(setup, dest):
-    _runsetup(setup, 'sdist', '-d', dest, '--formats=zip')
+    _runsetup(setup, 'sdist', '-d', dest)
 
 def bdist_egg(setup, executable, dest=None):
     # Backward compat:
@@ -196,6 +207,9 @@ def bdist_egg(setup, executable, dest=None):
         assert executable == sys.executable, (executable, sys.executable)
     _runsetup(setup, 'bdist_egg', '-d', dest)
 
+def bdist_wheel(setup, dest):
+    _runsetup(setup, 'bdist_wheel', '-d', dest)
+
 def wait_until(label, func, *args, **kw):
     if 'timeout' in kw:
         kw = dict(kw)
@@ -457,6 +471,8 @@ class Handler(BaseHTTPRequestHandler):
                 self.send_header('Content-Type', 'application/x-gzip')
             elif path.endswith('.zip'):
                 self.send_header('Content-Type', 'application/x-gzip')
+            elif path.endswith('.whl'):
+                self.send_header('Content-Type', 'application/octet-stream')
             else:
                 self.send_header('Content-Type', 'text/html')
 
@@ -595,7 +611,7 @@ normalize_exception_type_for_python_2_and_3 = (
 normalize_open_in_generated_script = (
     re.compile(r"open\(__file__, 'U'\)"), 'open(__file__)')
 
-not_found = (re.compile(r'Not found: [^\n]+/(\w|\.)+/\r?\n'), '')
+not_found = (re.compile(r'Not found: [^\n]+/(\w|\.|-)+/\r?\n'), '')
 
 easyinstall_deprecated = (re.compile(r'.*EasyInstallDeprecationWarning.*\n'),'')
 setuptools_deprecated = (re.compile(r'.*SetuptoolsDeprecationWarning.*\n'),'')
